from .. import Encoding, Offsets, Token
from typing import List, Optional, Union, Tuple, Dict

class Model:
    """Base class for all models

    This class is not supposed to be instantiated directly. Instead, any implementation of
    a Model will return a instance of this class when instantiated.
    """

    def tokenize(self, sequence: str) -> List[Token]:
        """ Tokenize the given sequence """
        pass
    def token_to_id(self, token: str) -> Optional[int]:
        """ Returns the id associated with the given token """
        pass
    def id_to_token(self, id: int) -> Optional[str]:
        """ Returns the token associated with the given id """
        pass
    def save(self, folder: str, name: Optional[str] = None) -> List[str]:
        """Save the current model

        Save the current model in the given folder, using the given name for the various
        files that will get created.
        Any file with the same name that already exist in this folder will be overwritten.
        """
        pass

class BPE(Model):
    """BytePairEncoding model class

    Instantiate a BPE Model from the given vocab and merges.

    Args:
       vocab: ('`optional`) Dict[str, int]:
           A dictionnary of string keys and their ids {"am": 0,...}

       merges: (`optional`) string:
           A list of pairs of tokens [("a", "b"),...]

       cache_capacity: (`optional`) int:
           The number of words that the BPE cache can contain. The cache allows
           to speed-up the process by keeping the result of the merge operations
           for a number of words.

       dropout: (`optional`) Optional[float] [0, 1]:
           The BPE dropout to use. Must be an float between 0 and 1

       unk_token: (`optional`) str:
           The unknown token to be used by the model.

       continuing_subword_prefix: (`optional`) str:
           The prefix to attach to subword units that don't represent a beginning of word.

       end_of_word_suffix: (`optional`) str:
           The suffix to attach to subword units that represent an end of word.

       fuse_unk: (`optional`) bool:
           Multiple unk tokens get fused into only 1
    """

    def __init__(
        self,
        vocab: Optional[Union[str, Dict[str, int]]],
        merges: Optional[Union[str, List[Tuple[str, str]]]],
        cache_capacity: Optional[int],
        dropout: Optional[float],
        unk_token: Optional[str],
        continuing_subword_prefix: Optional[str],
        end_of_word_suffix: Optional[str],
        fuse_unk: Optional[bool],
    ):
        pass
    @staticmethod
    def read_file(vocab_filename: str, merges_filename: str) -> Tuple[Vocab, Merges]:
        pass
    @staticmethod
    def from_file(vocab_filename: str, merges_filename: str, **kwargs) -> BPE:
        """
        Convenient method to intialize a BPE from files
        Roughly equivalent to

        def from_file(vocab_filename, merges_filenames, **kwargs):
            vocab, merges = BPE.read_file(vocab_filename, merges_filename)
            return BPE(vocab, merges, **kwargs)
        """
        pass

class WordPiece(Model):
    """WordPiece model class

    Instantiate a WordPiece Model from the given vocab file.

        Args:
            vocab: (`optional`) string:
                A dictionnary of string keys and their ids {"am": 0,...}

            unk_token: (`optional`) str:
                The unknown token to be used by the model.

            max_input_chars_per_word: (`optional`) int:
                The maximum number of characters to authorize in a single word.
    """

    def __init__(
        self,
        vocab: Optional[Union[str, Dict[str, int]]],
        unk_token: Optional[str],
        max_input_chars_per_word: Optional[int],
    ):
        pass
    @staticmethod
    def read_file(vocab_filename: str) -> Vocab:
        pass
    @staticmethod
    def from_file(vocab_filename: str, **kwargs) -> WordPiece:
        """
        Convenient method to intialize a WordPiece from file
        Roughly equivalent to

        def from_file(vocab_filename, **kwargs):
            vocab, merges = WordPiece.read_file(vocab_filename)
            return WordPiece(vocab, **kwargs)
        """
        pass

class WordLevel(Model):
    """
    Most simple tokenizer model based on mapping token from a vocab file to their corresponding id.

    Instantiate a WordLevel Model from the given vocab file.

        Args:
            vocab: (`optional`) string:
                A dictionnary of string keys and their ids {"am": 0,...}

            unk_token: str:
                The unknown token to be used by the model.
    """

    def __init__(self, vocab: Optional[Union[str, Dict[str, int]]], unk_token: Optional[str]):
        pass
    @staticmethod
    def read_file(vocab_filename: str) -> Vocab:
        pass
    @staticmethod
    def from_file(vocab_filename: str, **kwargs) -> WordLevelg:
        """
        Convenient method to intialize a WordLevelg from file
        Roughly equivalent to

        def from_file(vocab_filename, **kwargs):
            vocab, merges = WordLevelg.read_file(vocab_filename)
            return WordLevelg(vocab, **kwargs)
        """
        pass

class Unigram(Model):
    """UnigramEncoding model class

    Instantiate a Unigram Model from the given model file.

    Args:
       vocab: ('`optional`) string:
           A list of vocabulary items and their relative score [("am", -0.2442),...]

    """

    @staticmethod
    def __init__(self, vocab: Optional[List[Tuple[str, float]]]):
        pass



from .. import pre_tokenizers

PreTokenizer = pre_tokenizers.PreTokenizer
ByteLevel = pre_tokenizers.ByteLevel
Whitespace = pre_tokenizers.Whitespace
Punctuation = pre_tokenizers.Punctuation
Sequence = pre_tokenizers.Sequence
WhitespaceSplit = pre_tokenizers.WhitespaceSplit
BertPreTokenizer = pre_tokenizers.BertPreTokenizer
Metaspace = pre_tokenizers.Metaspace
CharDelimiterSplit = pre_tokenizers.CharDelimiterSplit
Digits = pre_tokenizers.Digits
UnicodeScripts = pre_tokenizers.UnicodeScripts



from typing import List

class Decoder:
    """Base class for all decoders

    This class is not supposed to be instantiated directly. Instead, any implementation of
    a Decoder will return an instance of this class when instantiated.
    """

    def decode(self, tokens: List[str]) -> str:
        """ Decode the given list of string to a final string """
        pass

class ByteLevel(Decoder):
    """ ByteLevel Decoder """

    def __init__(self) -> None:
        """ Instantiate a new ByteLevel Decoder """
        pass

class WordPiece(Decoder):
    """ WordPiece Decoder """

    @staticmethod
    def __init__(self, prefix: str = "##", cleanup: bool = True) -> Decoder:
        """Instantiate a new WordPiece Decoder

        Args:
            prefix: str:
                The prefix to use for subwords that are not a beginning-of-word
            cleanup: bool:
                Whether to cleanup some tokenization artifacts. Mainly spaces before punctuation,
                and some abbreviated english forms.
        """
        pass

class Metaspace(Decoder):
    """ Metaspace decoder """

    def __init__(self, replacement: str = "▁", add_prefix_space: bool = True) -> None:
        """Instantiate a new Metaspace

        Args:
            replacement: str:
                The replacement character. Must be exactly one character. By default we
                use the `▁` (U+2581) meta symbol (Same as in SentencePiece).

            add_prefix_space: boolean:
                Whether to add a space to the first word if there isn't already one. This
                lets us treat `hello` exactly like `say hello`.
        """
        pass

class BPEDecoder(Decoder):
    """ BPEDecoder """

    def __init__(self, suffix: str = "</w>") -> None:
        """Instantiate a new BPEDecoder

        Args:
            suffix: str:
                The suffix that was used to caracterize an end-of-word. This suffix will
                be replaced by whitespaces during the decoding
        """
        pass



from .. import NormalizedString
from typing import Optional, List

class Normalizer:
    """Base class for all normalizers

    This class is not supposed to be instantiated directly. Instead, any implementation of a
    Normalizer will return an instance of this class when instantiated.
    """

    def normalize(self, normalized: NormalizedString):
        """ Normalize the given NormalizedString in-place """
        pass
    def normalize_str(self, sequence: str) -> str:
        """ Normalize the given str """
        pass

class BertNormalizer(Normalizer):
    """BertNormalizer

    Takes care of normalizing raw text before giving it to a Bert model.
    This includes cleaning the text, handling accents, chinese chars and lowercasing
    """

    def __init__(
        self,
        clean_text: Optional[bool] = True,
        handle_chinese_chars: Optional[bool] = True,
        strip_accents: Optional[bool] = None,
        lowercase: Optional[bool] = True,
    ) -> None:
        """Instantiate a BertNormalizer with the given options.

        Args:
            clean_text: (`optional`) boolean:
                Whether to clean the text, by removing any control characters
                and replacing all whitespaces by the classic one.

            handle_chinese_chars: (`optional`) boolean:
                Whether to handle chinese chars by putting spaces around them.

            strip_accents: (`optional`) boolean:
                Whether to strip all accents. If this option is not specified (ie == None),
                then it will be determined by the value for `lowercase` (as in the original Bert).

            lowercase: (`optional`) boolean:
                Whether to lowercase.

        Returns:
            Normalizer
        """
        pass

class NFD(Normalizer):
    """ NFD Unicode Normalizer """

    def __init__(self) -> None:
        """ Instantiate a new NFD Normalizer """
        pass

class NFKD(Normalizer):
    """ NFKD Unicode Normalizer """

    def __init__(self) -> None:
        """ Instantiate a new NFKD Normalizer """
        pass

class NFC(Normalizer):
    """ NFC Unicode Normalizer """

    def __init__(self) -> None:
        """ Instantiate a new NFC Normalizer """
        pass

class NFKC(Normalizer):
    """ NFKC Unicode Normalizer """

    def __init__(self) -> None:
        """ Instantiate a new NFKC Normalizer """
        pass

class Sequence(Normalizer):
    """Allows concatenating multiple other Normalizer as a Sequence.

    All the normalizers run in sequence in the given order
    """

    def __init__(self, normalizers: List[Normalizer]) -> None:
        """Instantiate a new normalization Sequence using the given normalizers

        Args:
            normalizers: List[Normalizer]:
                A list of Normalizer to be run as a sequence
        """
        pass

class Lowercase(Normalizer):
    """ Lowercase Normalizer """

    def __init__(self) -> None:
        """ Instantiate a new Lowercase Normalizer """
        pass

class Strip(Normalizer):
    """ Strip normalizer """

    def __init__(self, left: bool = True, right: bool = True) -> Normalizer:
        pass

class StripAccents(Normalizer):
    """ StripAccents normalizer """

    def __init__(self) -> Normalizer:
        pass

class Nmt(Normalizer):
    """ Nmt normalizer """

    def __init__(self) -> Normalizer:
        pass

class Precompiled(Normalizer):
    """ Precompiled normalizer """

    def __init__(self, precompiled_charsmap: bytes) -> Normalizer:
        pass

class Replace(Normalizer):
    """ Replace normalizer """

    def __init__(self, pattern: str, content: str) -> Normalizer:
        pass

def unicode_normalizer_from_str(normalizer: str) -> Normalizer:
    """
    Instanciate unicode normalizer from the normalizer name
    :param normalizer: Name of the normalizer
    :return:
    """
    pass



from .. import AddedToken
from typing import Optional, List, Union

class Trainer:
    """Base class for all trainers

    This class is not supposed to be instantiated directly. Instead, any implementation of a
    Trainer will return an instance of this class when instantiated.
    """

class BpeTrainer(Trainer):
    """BpeTrainer

    Capable of training a BPE model
    """

    def __init__(
        self,
        vocab_size: int = 30000,
        min_frequency: int = 0,
        show_progress: bool = True,
        special_tokens: List[Union[str, AddedToken]] = [],
        limit_alphabet: Optional[int] = None,
        initial_alphabet: List[str] = [],
        continuing_subword_prefix: Optional[str] = None,
        end_of_word_suffix: Optional[str] = None,
    ) -> None:
        """Instantiate a new BpeTrainer with the given options:

        Args:
            vocab_size: unsigned int:
                The size of the final vocabulary, including all tokens and alphabet.

            min_frequency: unsigned int:
                The minimum frequency a pair should have in order to be merged.

            show_progress: boolean:
                Whether to show progress bars while training.

            special_tokens: List[Union[str, AddedToken]]:
                A list of special tokens the model should know of.

            limit_alphabet: unsigned int:
                The maximum different characters to keep in the alphabet.

            initial_alphabet: List[str]:
                A list of characters to include in the initial alphabet, even
                if not seen in the training dataset.
                If the strings contain more than one character, only the first one
                is kept.

            continuing_subword_prefix: Optional[str]:
                A prefix to be used for every subword that is not a beginning-of-word.

            end_of_word_suffix: Optional[str]:
                A suffix to be used for every subword that is a end-of-word.

        Returns:
            Trainer
        """
        pass

class WordPieceTrainer(Trainer):
    """WordPieceTrainer

    Capable of training a WordPiece model
    """

    def __init__(
        self,
        vocab_size: int = 30000,
        min_frequency: int = 0,
        show_progress: bool = True,
        special_tokens: List[Union[str, AddedToken]] = [],
        limit_alphabet: Optional[int] = None,
        initial_alphabet: List[str] = [],
        continuing_subword_prefix: Optional[str] = "##",
        end_of_word_suffix: Optional[str] = None,
    ) -> Trainer:
        """Instantiate a new WordPieceTrainer with the given options:

        Args:
            vocab_size: unsigned int:
                The size of the final vocabulary, including all tokens and alphabet.

            min_frequency: unsigned int:
                The minimum frequency a pair should have in order to be merged.

            show_progress: boolean:
                Whether to show progress bars while training.

            special_tokens: List[Union[str, AddedToken]]:
                A list of special tokens the model should know of.

            limit_alphabet: unsigned int:
                The maximum different characters to keep in the alphabet.

            initial_alphabet: List[str]:
                A list of characters to include in the initial alphabet, even
                if not seen in the training dataset.
                If the strings contain more than one character, only the first one
                is kept.

            continuing_subword_prefix: Optional[str]:
                A prefix to be used for every subword that is not a beginning-of-word.

            end_of_word_suffix: Optional[str]:
                A suffix to be used for every subword that is a end-of-word.

        Returns:
            Trainer
        """
        pass

class UnigramTrainer(Trainer):
    """UnigramTrainer

    Capable of training a Unigram model
    """

    def __init__(
        self,
        vocab_size: int = 8000,
        show_progress: bool = True,
        special_tokens: List[Union[str, AddedToken]] = [],
    ) -> Trainer:
        """Instantiate a new UnigramTrainer with the given options:

        Args:
            vocab_size: unsigned int:
                The size of the final vocabulary, including all tokens and alphabet.

            show_progress: boolean:
                Whether to show progress bars while training.

            special_tokens: List[Union[str, AddedToken]]:
                A list of special tokens the model should know of.

            initial_alphabet: List[str]:
                A list of characters to include in the initial alphabet, even
                if not seen in the training dataset.
                If the strings contain more than one character, only the first one
                is kept.

        Returns:
            Trainer
        """
        pass



from .. import processors

PostProcessor = processors.PostProcessor
BertProcessing = processors.BertProcessing
RobertaProcessing = processors.RobertaProcessing
ByteLevel = processors.ByteLevel
TemplateProcessing = processors.TemplateProcessing



from .. import PreTokenizedString
from typing import Optional, List, Tuple

Offsets = Tuple[int, int]

class PreTokenizer:
    """Base class for all pre-tokenizers

    This class is not supposed to be instantiated directly. Instead, any implementation of a
    PreTokenizer will return an instance of this class when instantiated.
    """

    def pre_tokenize(self, pretokenized: PreTokenizedString):
        """ Pre tokenize the given PreTokenizedString in-place """
        pass
    def pre_tokenize_str(self, sequence: str) -> List[Tuple[str, Offsets]]:
        """ Pre tokenize the given sequence """
        pass

class ByteLevel(PreTokenizer):
    """ByteLevel PreTokenizer

    This pre-tokenizer takes care of replacing all bytes of the given string
    with a corresponding representation, as well as splitting into words.
    """

    def __init__(self, add_prefix_space: bool = True) -> None:
        """Instantiate a new ByteLevel PreTokenizer
        Args:
            add_prefix_space: (`optional`) boolean:
                Whether to add a space to the first word if there isn't already one. This
                lets us treat `hello` exactly like `say hello`.
        Returns:
            PreTokenizer
        """
        pass
    @staticmethod
    def alphabet() -> List[str]:
        """Returns the alphabet used by this PreTokenizer.

        Since the ByteLevel works as its name suggests, at the byte level, it
        encodes any byte to one visible character. This means that there is a
        total of 256 different characters composing this alphabet.
        """
        pass

class Whitespace(PreTokenizer):
    """Whitespace PreTokenizer

    This pre-tokenizer simply splits using the following regex: `\w+|[^\w\s]+`
    """

    def __init__(self) -> None:
        """ Instantiate a new Whitespace PreTokenizer """
        pass

class WhitespaceSplit(PreTokenizer):
    """Whitespace PreTokenizer

    This pre-tokenizer simply splits on the whitespace. Works like `.split()`
    """

    def __init__(self) -> None:
        """ Instantiate a new WhitespaceSplit PreTokenizer """
        pass

class BertPreTokenizer(PreTokenizer):
    """BertPreTokenizer

    This pre-tokenizer splits tokens on spaces, and also on punctuation.
    Each occurence of a punctuation character will be treated separately.
    """

    def __init__(self) -> None:
        """ Instantiate a new BertPreTokenizer """
        pass

class Metaspace(PreTokenizer):
    """Metaspace pre-tokenizer

    This pre-tokenizer replaces any whitespace by the provided replacement character.
    It then tries to split on these spaces.
    """

    def __init__(self, replacement: str = "▁", add_prefix_space: bool = True) -> None:
        """Instantiate a new Metaspace

        Args:
            replacement: str:
                The replacement character. Must be exactly one character. By default we
                use the `▁` (U+2581) meta symbol (Same as in SentencePiece).

            add_prefix_space: boolean:
                Whether to add a space to the first word if there isn't already one. This
                lets us treat `hello` exactly like `say hello`.
        """
        pass

class CharDelimiterSplit(PreTokenizer):
    """CharDelimiterSplit PreTokenizer

    This pre-tokenizer simply splits on the provided char. Works like `.split(delimiter)`
    """

    @staticmethod
    def __init__(self, delimiter: str) -> None:
        """Instantiate a new CharDelimiterSplit PreTokenizer

        Args:
            delimiter: str:
                The delimiter char that will be used to split input
        """
        pass

class Punctuation(PreTokenizer):
    """Punctuation PreTokenizer

    This pre-tokenizer simply splits on punctuation as individual characters.`
    """

    def __init__(self) -> None:
        """ Instantiate a new Punctuation PreTokenizer """
        pass

class Sequence(PreTokenizer):
    """Sequence PreTokenizer

    This pre-tokenizer composes other pre_tokenizers and applies them in sequence`
    """

    def __init__(self) -> None:
        """ Instantiate a new Sequence PreTokenizer """
        pass

class Digits(PreTokenizer):
    """Digits PreTokenizer

    This pre-tokenizer simply splits using the digits in separate tokens
    """

    def __init__(self, individual_digits: bool) -> None:
        """Instantiate a new Digits

        Args:
            individual_digits: bool:
                If set to True, digits will each be separated "Call 123 please" -> "Call ", "1", "2", "3", " please"
                If set to False, digits will grouped "Call 123 please" -> "Call ", "123", " please"

        """
        pass

class UnicodeScripts(PreTokenizer):
    """UnicodeScripts PreTokenizer

    This pre-tokenizer splits on characters that belong to different language family
    It roughly follows https://github.com/google/sentencepiece/blob/master/data/Scripts.txt
    Actually Hiragana and Katakana are fused with Han, and 0x30FC is Han too.
    This mimicks SentencePiece Unigram implementation.
    """

    def __init__(self) -> None:
        """ Instantiate a new UnicodeScripts """
        pass



from .. import Encoding
from typing import Tuple, Union, List

class PostProcessor:
    """Base class for all post-processors

    This class is not supposed to be instantiated directly. Instead, any implementation of
    a PostProcessor will return an instance of this class when instantiated.
    """

    def num_special_tokens_to_add(self, is_pair: bool) -> int:
        """
        Return the number of special tokens that would be added for single/pair sentences.
        :param is_pair: Boolean indicating if the input would be a single sentence or a pair
        :return:
        """
        pass
    def process(
        self, encoding: Encoding, pair: Optional[Encoding] = None, add_special_tokens: bool = True
    ) -> Encoding:
        """ Post-process the given encodings, generating the final one """
        pass

class BertProcessing(PostProcessor):
    """BertProcessing

    This post-processor takes care of adding the special tokens needed by
    a Bert model:
        - a SEP token
        - a CLS token
    """

    def __init__(self, sep: Tuple[str, int], cls: Tuple[str, int]) -> None:
        """Instantiate a new BertProcessing with the given tokens

        Args:
            sep: Tuple[str, int]:
                A tuple with the string representation of the SEP token, and its id

            cls: Tuple[str, int]:
                A tuple with the string representation of the CLS token, and its id

        Returns:
            PostProcessor
        """
        pass

class RobertaProcessing(PostProcessor):
    """RobertaProcessing

    This post-processor takes care of adding the special tokens needed by
    a Roberta model:
        - a SEP token
        - a CLS token

    It also takes care of trimming the offsets.
    By default, the ByteLevel BPE might include whitespaces in the produced tokens. If you don't
    want the offsets to include these whitespaces, then this PostProcessor should be initialized
    with `trim_offsets=True`
    """

    def __init__(
        self,
        sep: Tuple[str, int],
        cls: Tuple[str, int],
        trim_offsets: bool = True,
        add_prefix_space: bool = True,
    ) -> None:
        """Instantiate a new RobertaProcessing with the given tokens

        Args:
            sep: Tuple[str, int]:
                A tuple with the string representation of the SEP token, and its id

            cls: Tuple[str, int]:
                A tuple with the string representation of the CLS token, and its id

            trim_offsets: bool:
                Whether to trim the whitespaces from the produced offsets.

            add_prefix_space: bool:
                Whether the add_prefix_space option was enabled during pre-tokenization. This
                is relevant because it defines the way the offsets are trimmed out.

        Returns:
            PostProcessor
        """
        pass

class ByteLevel(PostProcessor):
    """ByteLevel Post processing

    This post-processor takes care of trimming the offsets.
    By default, the ByteLevel BPE might include whitespaces in the produced tokens. If you don't
    want the offsets to include these whitespaces, then this PostProcessor must be used.
    """

    def __init__(self, trim_offsets: bool = True) -> None:
        """Instantiate a new ByteLevel

        Args:
            trim_offsets: bool:
                Whether to trim the whitespaces from the produced offsets.
        """
        pass

Template = Union[str, List[str]]
Tokens = List[Union[Tuple[int, str], Tuple[str, int], dict]]

class TemplateProcessing(PostProcessor):
    """TemplateProcessing

    Provides a way to specify templates in order to add the special tokens to each
    input sequence as relevant.

    Let's take `BERT` tokenizer as an example. It uses two special tokens, used to
    delimitate each sequence. `[CLS]` is always used at the beginning of the first
    sequence, and `[SEP]` is added at the end of both the first, and the pair
    sequences. The final result looks like this:
        - Single sequence: `[CLS] Hello there [SEP]`
        - Pair sequences: `[CLS] My name is Anthony [SEP] What is my name? [SEP]`
    With the type ids as following:
    ```markdown
    [CLS]   ...   [SEP]   ...   [SEP]
      0      0      0      1      1
    ```

    You can achieve such behavior using a TemplateProcessing:
    ```
    TemplateProcessing(
        single="[CLS] $0 [SEP]",
        pair="[CLS] $A [SEP] $B:1 [SEP]:1",
        special_tokens=[("[CLS]", 1), ("[SEP]", 0)],
    )
    ```

    In this example, each input sequence is identified using a `$` construct. This identifier
    lets us specify each input sequence, and the type_id to use. When nothing is specified,
    it uses the default values. Here are the different ways to specify it:
    - Specifying the sequence, with default `type_id == 0`: `$A` or `$B`
    - Specifying the `type_id` with default `sequence == A`: `$0`, `$1`, `$2`, ...
    - Specifying both: `$A:0`, `$B:1`, ...

    The same construct is used for special tokens: `<identifier>(:<type_id>)?`.

    **Warning**: You must ensure that you are giving the correct tokens/ids as these
    will be added to the Encoding without any further check. If the given ids correspond
    to something totally different in a `Tokenizer` using this `PostProcessor`, it
    might lead to unexpected results.
    """

    def __init__(self, single: Template, pair: Template, special_tokens: Tokens) -> None:
        """Instantiate a new TemplateProcessing

        Args:
            single: Template
                The template used for single sequences

            pair: Template:
                The template used when both sequences are specified

            special_tokens: Tokens:
                The list of special tokens used in each sequences

        Template: Union[str, List[str]]:
            - If a `str` is provided, the whitespace is used as delimiter between tokens
            - If a `List[str]` is provided, a list of tokens

        Tokens: List[Union[Tuple[int, str], Tuple[str, int], dict]]:
            - A Tuple with both a token and its associated ID, in any order
            - A dict with the following keys:
                - "id": str => The special token id, as specified in the Template
                - "ids": List[int] => The associated IDs
                - "tokens": List[str] => The associated tokens
             The given dict expects the provided `ids` and `tokens` lists to have
             the same length.
        """
        pass
